package com.tzld.piaoquan.ad.engine.commons.score.model;


import com.tzld.piaoquan.ad.engine.commons.util.CompressUtil;
import com.tzld.piaoquan.ad.engine.commons.util.PropertiesUtil;
import ml.dmlc.xgboost4j.scala.DMatrix;
import ml.dmlc.xgboost4j.scala.spark.XGBoostClassificationModel;
import org.apache.commons.lang.math.NumberUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.file.Paths;
import java.util.Map;


public class XGBoostModel extends Model {
    private static final Logger LOGGER = LoggerFactory.getLogger(XGBoostModel.class);
    private XGBoostClassificationModel model;

    private String[] features = {
            "cpa",
            "b2_1h_ctr",
            "b2_1h_ctcvr",
            "b2_1h_cvr",
            "b2_1h_conver",
            "b2_1h_click",
            "b2_1h_conver*log(view)",
            "b2_1h_conver*ctcvr",
            "b2_2h_ctr",
            "b2_2h_ctcvr",
            "b2_2h_cvr",
            "b2_2h_conver",
            "b2_2h_click",
            "b2_2h_conver*log(view)",
            "b2_2h_conver*ctcvr",
            "b2_3h_ctr",
            "b2_3h_ctcvr",
            "b2_3h_cvr",
            "b2_3h_conver",
            "b2_3h_click",
            "b2_3h_conver*log(view)",
            "b2_3h_conver*ctcvr",
            "b2_6h_ctr",
            "b2_6h_ctcvr"
    };

    @Override
    public int getModelSize() {
        if (this.model == null)
            return 0;
        return 1;
    }

    @Override
    public boolean loadFromStream(InputStreamReader in) throws Exception {
        return false;
    }

    public void cleanModel() {
        this.model = null;
    }

    public Float score(Map<String, String> featureMap) {

        try {
            float[] values = new float[features.length];
            for (int i = 0; i < features.length; i++) {
                float v = NumberUtils.toFloat(featureMap.getOrDefault(features[i], "0.0"), 0.0f);
                values[i] = v;
            }
            DMatrix dm = new DMatrix(values, 1, features.length, 0.0f);
            float[][] result = model._booster().predict(dm, false, 100);
            return result[0][0];
        } catch (Exception e) {
            return 0f;
        }
    }

    @Override
    public boolean loadFromStream(InputStream in) throws Exception {
        String modelDir = PropertiesUtil.getString("model.xgboost.path");
        CompressUtil.decompressGzFile(in, modelDir);
        String absolutePath =new File(rpath).getAbsolutePath();
        XGBoostClassificationModel model2 = XGBoostClassificationModel.load("file://" + absolutePath);
        model2.setMissing(0.0f);
        this.model = model2;
        return true;
    }

}
